@@ -83,7 +83,8 @@ def moving_window_startstop(start, stop, step, window=None):
     # Reduce `stop` to an integer number of `step` and adjust for last window
     stop_corrected = (stop - stop % step) - window + step
     # differently from range, np.arange accepts also floats
-    return [(t1, t1+window) for t1 in np.arange(start, stop_corrected, step)]
+    return [(t1, t1 + window) for t1 in np.arange(start, stop_corrected, step)]
+
 
 def moving_window_dataframe(start, stop, step, window=None):
     """Create a DataFrame for storing moving-window data.
@@ -113,6 +114,7 @@ def moving_window_dataframe(start, stop, step, window=None):
     df = pd.DataFrame(data=dict(tstart=tstart, tstop=tstop, tmean=tmean))
     return df
 
+
 def moving_window_chunks(dx, start, stop, step, window=None,
                          time_zero=0):
     """Return a list of Data object, each containing bursts in one time-window.
@@ -162,7 +164,7 @@ def calc_mean_lifetime(dx, t1=0, t2=np.inf, ph_sel=Ph_sel('all')):
 
     for bursts, nanot, mask in zip(dx.mburst, dx.nanotimes,
                                    dx.iter_ph_masks(ph_sel)):
-        selection = (nanot > t1)*(nanot < t2)
+        selection = (nanot > t1) * (nanot < t2)
         # Select photons in ph_sel AND with nanotime in [t1, t2]
         if isarray(mask):
             selection *= mask
@@ -191,6 +193,7 @@ def _store_bg_data(store, base_name, min_ph_delays_us, best_bg, best_th,
     store[base_name + 'best_th'] = best_th
     store.close()
 
+
 def _load_bg_data(store, base_name, ph_streams):
     if not base_name.endswith('/'):
         base_name = base_name + '/'
@@ -210,6 +213,7 @@ def _load_bg_data(store, base_name, ph_streams):
     store.close()
     return best_th, best_bg, BG_data, BG_data_e, min_ph_delays
 
+
 def calc_bg_brute_cache(dx, min_ph_delay_list=None, return_all=False,
                         error_metrics='KS', force_recompute=False):
     """Compute background for all the ch, ph_sel and periods caching results.
@@ -267,6 +271,7 @@ def calc_bg_brute_cache(dx, min_ph_delay_list=None, return_all=False,
     else:
         return res[:-3]
 
+
 def calc_bg_brute(dx, min_ph_delay_list=None, return_all=False,
                   error_metrics='KS'):
     """Compute background for all the ch, ph_sel and periods.
@@ -300,7 +305,7 @@ def calc_bg_brute(dx, min_ph_delay_list=None, return_all=False,
     BG_data, BG_data_e = {}, {}
 
     index = pd.MultiIndex.from_product([list(range(dx.nch)), ph_sel_labels],
-                                        names=['CH', 'ph_sel'])
+                                       names=['CH', 'ph_sel'])
     best_th = pd.DataFrame(index=index, columns=np.arange(dx.nperiods))
     best_th.columns.name = 'period'
     best_th.sortlevel(inplace=True)
@@ -319,33 +324,37 @@ def calc_bg_brute(dx, min_ph_delay_list=None, return_all=False,
                 b = BG_data_e[ph_sel][ich, period, :]
                 i_best_th = b[-np.isnan(b)].argmin()
                 best_th.loc[(ich, str(ph_sel)), period] = \
-                        min_ph_delay_list[i_best_th]
+                    min_ph_delay_list[i_best_th]
                 best_bg.loc[(ich, str(ph_sel)), period] = \
-                        BG_data[ph_sel][ich, period, i_best_th]
+                    BG_data[ph_sel][ich, period, i_best_th]
     if return_all:
         return best_th, best_bg, BG_data, BG_data_e, min_ph_delay_list
     else:
         return best_th, best_bg
 
 
-def burst_data(dx, ich=0, include_bg=False, include_ph_index=False):
+def burst_data(dx, include_bg=False, include_ph_index=False,
+               skip_ch=None):
     """Return a table (`pd.DataFrame`) of burst data (one row per burst).
 
     Columns include:
 
-    - *nd*, *na*, *naa*: burst counts in DexDem, DexAem, AexAem
-      photon streams.
+    - *E* and *S*: FRET and stoichiometry for each burst.
+    - *nd*, *na*, *naa*, *nda*: burst counts in DexDem, DexAem, AexAem and
+      AexDem photon streams.
     - *t_start*, *t_stop*: time (in seconds) of first and last photon inside
       the burst
     - *width_ms*: burst duration in milliseconds
-    - *size_raw*: uncorrected total counts in the burst
+    - *size_raw*: the total uncorrected burst counts in the photon stream
+      used for burst search
 
     Optional columns include:
 
     - *i_start*, *i_stop*: index of burst start and stop relative to the
       original timestamps array (requires `include_ph_index=True`)
-    - *bg_dd*, *bg_ad*, *bg_aa*: background contribution in the DexDem, DexAem,
-      AexAem photon stream (requires `include_bg=True`)
+    - *bg_dd*, *bg_ad*, *bg_aa*, *bg_da*: background contribution in the
+      DexDem, DexAem, AexAem, AexDem photon streams (requires
+      `include_bg=True`).
 
     If the peak photon-counts in each bursts has been computed (see
     :meth:`fretbursts.burstlib.Data.calc_max_rate`), it will
@@ -356,43 +365,35 @@ def burst_data(dx, ich=0, include_bg=False, include_ph_index=False):
             background (see above). Default False.
         include_ph_index (bool): if True includes additional two columns for
             index of first and last timestamp in each burst. Default False.
+        skip_ch (list or None): List of channels to skip if measurement is
+            multispot.
 
     Return:
         A pandas's DataFrame containing burst data (one row per burst).
     """
-    if dx.ALEX:
-        nd, na, naa, bg_d, bg_a, bg_aa, wid = dx.expand(ich=ich, alex_naa=True,
-                                                        width=True)
-        nt = nd + na + naa + dx.nda[ich]
+    def _burst_data_ich(dx, ich, include_bg=False, include_ph_index=False):
+        bursts_ich = dx.burst_data_ich(ich=ich)
+        if not include_ph_index:
+            bursts_ich.pop('i_start')
+            bursts_ich.pop('i_stop')
+        if not include_bg:
+            bursts_ich = {k: v for k, v in bursts_ich.items()
+                          if not k.startswith('bg')}
+        return pd.DataFrame.from_dict(bursts_ich)
+
+    if skip_ch is None:
+        skip_ch = []
+    kws = dict(include_bg=include_bg, include_ph_index=include_ph_index)
+    bursts = {}
+    for ich in range(dx.nch):
+        if ich not in skip_ch:
+            bursts[ich] = _burst_data_ich(dx, ich=ich, **kws)
+            bursts[ich]['spot'] = ich
+    if dx.nch > 1:
+        bursts = pd.concat(bursts, ignore_index=True)
     else:
-        nd, na, bg_d, bg_a, wid = dx.expand(ich=ich, width=True)
-        nt = nd + na
-
-    size_raw = dx.mburst[ich].counts
-    t_start = dx.mburst[ich].start * dx.clk_p
-    t_end = dx.mburst[ich].stop * dx.clk_p
-    i_start = dx.mburst[ich].istart
-    i_end = dx.mburst[ich].istop
-    #asym = asymmetry(dx, dropnan=False)
-
-    data_dict = dict(size_raw=size_raw, nt=nt, width_ms=wid*1e3,
-                     t_start=t_start, t_end=t_end)#, asymmetry=asym)
-
-    if include_ph_index:
-        data_dict.update(i_start=i_start, i_end=i_end)
-
-    if include_bg:
-        data_dict.update(bg_d=bg_d, bg_a=bg_a)
-        if dx.ALEX:
-            data_dict.update(bg_aa=bg_aa)
-
-    burst_fields = dx.burst_fields[:]
-    burst_fields.remove('mburst')
-    for field in burst_fields:
-        if field in dx:
-            data_dict[field] = dx[field][ich]
-
-    return pd.DataFrame.from_dict(data_dict)
+        bursts = bursts[0]
+    return bursts
 
 
 def fit_bursts_kde_peak(dx, burst_data='E', bandwidth=0.03, weights=None,
@@ -431,7 +432,7 @@ def fit_bursts_kde_peak(dx, burst_data='E', bandwidth=0.03, weights=None,
     return KDE_max_mch
 
 def bursts_fitter(dx, burst_data='E', save_fitter=True,
-                  weights=None, gamma=1, add_naa=False,
+                  weights=None, gamma=1, add_naa=False, skip_ch=None,
                   binwidth=None, bandwidth=None, model=None, verbose=False):
     """Create a mfit.MultiFitter object (for E or S) add it to `dx`.
 
@@ -459,7 +460,7 @@ def bursts_fitter(dx, burst_data='E', save_fitter=True,
         The `mfit.MultiFitter` object with the specified burst-size weights.
     """
     assert burst_data in dx
-    fitter = mfit.MultiFitter(dx[burst_data])
+    fitter = mfit.MultiFitter(dx[burst_data], skip_ch=skip_ch)
     if weights is None or _is_list_of_arrays(weights):
         # If no weights or precomputed weights
         fitter.weights = weights
@@ -777,6 +778,7 @@ def join_data(d_list, gap=0):
 
     return new_d
 
+
 def burst_search_and_gate(dx, F=6, m=10, min_rate_cps=None, c=-1,
                           ph_sel1=Ph_sel(Dex='DAem'),
                           ph_sel2=Ph_sel(Aex='Aem'), compact=False, mute=False):
@@ -814,6 +816,7 @@ def burst_search_and_gate(dx, F=6, m=10, min_rate_cps=None, c=-1,
     dx_d = dx.copy(mute=mute)
     dx_a = dx.copy(mute=mute)
     dx_and = dx.copy(mute=mute)
+    dx_and.delete_burst_data()
 
     dx_d.burst_search(L=m, m=m, F=F, min_rate_cps=min_rate_cps, c=c,
                       ph_sel=ph_sel1, compact=compact, computefret=False,
