@@ -19,26 +19,17 @@ from builtins import range, zip
 
 import os
 import numpy as np
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
 import tables
 
-from .dataload.multi_ch_reader import load_data_ordered16
-from .dataload.smreader import load_sm
+from phconvert.smreader import load_sm
 from .dataload.spcreader import load_spc
-from .dataload.manta_reader import (load_manta_timestamps,
-                                    load_xavier_manta_data,
-                                    get_timestamps_detectors,
-                                    # process_timestamps,
-                                    process_store,
-                                    load_manta_timestamps_pytables)
-from .utils.misc import pprint, deprecate
 from .burstlib import Data
 from . import loader_legacy
 import phconvert as phc
 
+import logging
+log = logging.getLogger(__name__)
+
 
 def _is_multich(h5data):
     if 'photon_data' in h5data:
@@ -49,12 +40,14 @@ def _is_multich(h5data):
         msg = 'Cannot find a photon_data group.'
         raise phc.hdf5.Invalid_PhotonHDF5(msg)
 
+
 def _append_data_ch(d, name, value):
     if name not in d:
         d.add(**{name: [value]})
     else:
         d[name].append(value)
 
+
 def _load_from_group(d, group, name, dest_name, multich_field=False,
                      ondisk=False, allow_missing=True):
     if allow_missing and name not in group:
@@ -68,6 +61,7 @@ def _load_from_group(d, group, name, dest_name, multich_field=False,
     else:
         d.add(**{dest_name: node_value})
 
+
 def _append_empy_ch(data):
     # Empty channel, fill it with empty arrays
     ph_times = np.array([], dtype='int64')
@@ -76,7 +70,8 @@ def _append_empy_ch(data):
     a_em = np.array([], dtype=bool)
     _append_data_ch(data, 'A_em', a_em)
 
-def _get_measurement_specs(ph_data):
+
+def _get_measurement_specs(ph_data, setup):
     if 'measurement_specs' not in ph_data:
         # No measurement specs, we will load timestamps and set them all in a
         # conventional photon stream (acceptor emission)
@@ -88,11 +83,34 @@ def _get_measurement_specs(ph_data):
         meas_type = meas_specs.measurement_type.read().decode()
 
     if meas_type not in ['smFRET-1color', 'smFRET',
-                         'smFRET-usALEX', 'smFRET-nsALEX']:
+                         'smFRET-usALEX', 'smFRET-nsALEX', 'generic']:
         raise NotImplementedError('Measurement type "%s" not supported'
                                   ' by FRETBursts.' % meas_type)
+    if meas_type == 'generic':
+        msg = ('This file contains {n} {type} channels.\n'
+               'Unfortunately, the current FRETBursts version only supports\n'
+               '{nvalid} {type} channel.')
+        if setup.num_polarization_ch.read() != 1:
+            raise ValueError(msg.format(n=setup.num_polarization_ch.read(),
+                                        type='polarization', nvalid=1))
+        if setup.num_split_ch.read() != 1:
+            raise ValueError(msg.format(n=setup.num_split_ch.read(),
+                                        type='split', nvalid=1))
+        if setup.num_spectral_ch.read() != 2:
+            raise ValueError(msg.format(n=setup.num_spectral_ch.read(),
+                                        type='spectral', nvalid=2))
+        if not setup.modulated_excitation.read():
+            meas_type = 'smFRET'
+        elif tuple(setup.excitation_alternated.read()) == (False, True):
+            meas_type = 'PAX'
+        elif tuple(setup.excitation_alternated.read()) == (True, True):
+            if setup.lifetime.read():
+                meas_type = 'smFRET-nsALEX'
+            else:
+                meas_type = 'smFRET-usALEX'
     return meas_type, meas_specs
 
+
 def _load_photon_data_arrays(data, ph_data, meas_type, ondisk=False):
     assert 'timestamps' in ph_data
 
@@ -100,7 +118,7 @@ def _load_photon_data_arrays(data, ph_data, meas_type, ondisk=False):
     # fields not mapped use the same name on both Photon-HDF5 and FRETBursts
     mapping = {'timestamps': 'ph_times_m',
                'nanotimes': 'nanotimes', 'particles': 'particles'}
-    if 'ALEX' in meas_type:
+    if 'ALEX' in meas_type or 'PAX' in meas_type:
         mapping = {'timestamps': 'ph_times_t', 'detectors': 'det_t',
                    'nanotimes': 'nanotimes_t', 'particles': 'particles_t'}
 
@@ -113,6 +131,7 @@ def _load_photon_data_arrays(data, ph_data, meas_type, ondisk=False):
     # Timestamps are always present, and their units are always present too
     data.add(clk_p=ph_data.timestamps_specs.timestamps_unit.read())
 
+
 def _load_nanotimes_specs(data, ph_data):
     nanot_specs = ph_data.nanotimes_specs
     nanotimes_params = {}
@@ -127,6 +146,18 @@ def _load_nanotimes_specs(data, ph_data):
                 nanotimes_params.update(**{name: value})
     _append_data_ch(data, 'nanotimes_params', nanotimes_params)
 
+
+def _add_usALEX_specs(data, meas_specs):
+    try:
+        offset = meas_specs.alex_offset.read()
+    except tables.NoSuchNodeError:
+        log.warning('    No offset found, assuming offset = 0.')
+        offset = 0
+    data.add(offset=offset)
+    data.add(alex_period=meas_specs.alex_period.read())
+    _load_alex_periods_donor_acceptor(data, meas_specs)
+
+
 def _load_alex_periods_donor_acceptor(data, meas_specs):
     # Both us- and ns-ALEX
     try:
@@ -135,10 +166,20 @@ def _load_alex_periods_donor_acceptor(data, meas_specs):
         A_ON = meas_specs.alex_excitation_period2.read()
     except tables.NoSuchNodeError:
         # But if it fails it's OK, those fields are optional
-        print('WARNING: No alternation defintion found.')
+        msg = """
+        The current file lacks the alternation period defintion.
+        You will need to manually add this info using:
+
+          d.add(D_ON=D_ON, A_ON=A_ON)
+
+        where `d` is a Data object and D_ON/A_ON is a tuple with start/stop
+        values defining the D/A excitation excitation period. Values are in
+        raw timestamps units.
+        """
+        log.warning(msg)
     else:
-        _append_data_ch(data, 'D_ON', D_ON)
-        _append_data_ch(data, 'A_ON', A_ON)
+        data.add(D_ON=D_ON, A_ON=A_ON)
+
 
 def _compute_acceptor_emission_mask(data, ich, ondisk):
     """For non-ALEX measurements."""
@@ -165,16 +206,8 @@ def _compute_acceptor_emission_mask(data, ich, ondisk):
         # Create the boolean mask
         _append_data_ch(data, 'A_em', data.detectors[ich][:] == accept)
 
-def _add_usALEX_specs(data, meas_specs):
-    try:
-        offset = meas_specs.alex_offset.read()
-    except tables.NoSuchNodeError:
-        print('WARNING: No offset found, assuming offset = 0.')
-        offset = 0
-    data.add(offset=offset)
-    data.add(alex_period=meas_specs.alex_period.read())
 
-def _photon_hdf5_1ch(h5data, data, ondisk=False, nch=1, ich=0):
+def _photon_hdf5_1ch(h5data, data, ondisk=False, nch=1, ich=0, loadspecs=True):
     data.add(nch=nch)
     ph_data_name = '/photon_data' if nch == 1 else '/photon_data%d' % ich
 
@@ -185,7 +218,7 @@ def _photon_hdf5_1ch(h5data, data, ondisk=False, nch=1, ich=0):
 
     # Load photon_data group and measurement_specs (if present)
     ph_data = h5data._f_get_child(ph_data_name)
-    meas_type, meas_specs = _get_measurement_specs(ph_data)
+    meas_type, meas_specs = _get_measurement_specs(ph_data, h5data.setup)
 
     # Load photon_data arrays
     _load_photon_data_arrays(data, ph_data, meas_type=meas_type, ondisk=ondisk)
@@ -200,10 +233,6 @@ def _photon_hdf5_1ch(h5data, data, ondisk=False, nch=1, ich=0):
         accept = np.asscalar(meas_specs.detectors_specs.spectral_ch2.read())
         _append_data_ch(data, 'det_donor_accept', (donor, accept))
 
-    # Load alternation definition both for ns-ALEX and us-ALEX
-    if 'ALEX' in meas_type:
-        _load_alex_periods_donor_acceptor(data, meas_specs)
-
     # Here there are all the special-case for each measurement type
     if meas_type == 'smFRET-1color':
         # Non-FRET or unspecified data, assume all photons are "acceptor"
@@ -212,24 +241,27 @@ def _photon_hdf5_1ch(h5data, data, ondisk=False, nch=1, ich=0):
     elif meas_type == 'smFRET':
         _compute_acceptor_emission_mask(data, ich, ondisk=ondisk)
 
-    elif meas_type == 'smFRET-usALEX':
+    elif loadspecs and (meas_type == 'smFRET-usALEX' or 'PAX' in meas_type):
         _add_usALEX_specs(data, meas_specs)
 
-    elif meas_type == 'smFRET-nsALEX':
+    elif loadspecs and meas_type == 'smFRET-nsALEX':
         data.add(laser_repetition_rate=meas_specs.laser_repetition_rate.read())
+        _load_alex_periods_donor_acceptor(data, meas_specs)
 
     # Set some `data` flags
+    data.add(meas_type=meas_type)
     data.add(ALEX='ALEX' in meas_type)
+    data.add(alternated=data.ALEX or 'PAX' in data.meas_type)
     data.add(lifetime='nanotimes' in ph_data)
 
 
 def _photon_hdf5_multich(h5data, data, ondisk=True):
-
     ph_times_dict = phc.hdf5.photon_data_mapping(h5data._v_file)
     nch = np.max(list(ph_times_dict.keys())) + 1
-
-    for ich in range(nch):
-        _photon_hdf5_1ch(h5data, data, ondisk=ondisk, nch=nch, ich=ich)
+    _photon_hdf5_1ch(h5data, data, ondisk=ondisk, nch=nch, ich=0)
+    for ich in range(1, nch):
+        _photon_hdf5_1ch(h5data, data, ondisk=ondisk, nch=nch, ich=ich,
+                         loadspecs=False)
 
 
 def photon_hdf5(filename, ondisk=False, strict=False):
@@ -278,110 +310,6 @@ def photon_hdf5(filename, ondisk=False, strict=False):
 ##
 # Multi-spot loader functions
 #
-def multispot8(fname, bytes_to_read=-1, swap_D_A=True, leakage=0, gamma=1.):
-    """Load a 8-ch multispot file and return a Data() object. Cached version.
-    """
-    fname_c = fname + '_cache.pickle'
-    try:
-        var = pickle.load(open(fname_c, 'rb'))
-        dx = Data(fname=fname, clk_p=12.5e-9, nch=8, leakage=leakage,
-                  gamma=gamma)
-        dx.add(ph_times_m=var['ph_times_m'], A_em=var['A_em'], ALEX=False)
-        pprint(" - File loaded from cache: %s\n" % fname)
-    except IOError:
-        dx = multispot8_core(fname, bytes_to_read=bytes_to_read,
-                             swap_D_A=swap_D_A, leakage=leakage, gamma=gamma)
-        D = {'ph_times_m': dx.ph_times_m, 'A_em': dx.A_em}
-        pprint(" - Pickling data ... ")
-        pickle.dump(D, open(fname_c, 'wb'), -1)
-        pprint("DONE\n")
-    return dx
-
-load_multispot8 = deprecate(multispot8, "load_multispot8", "loader.multispot8")
-
-def multispot8_core(fname, bytes_to_read=-1, swap_D_A=True, leakage=0,
-                    gamma=1.):
-    """Load a 8-ch multispot file and return a Data() object.
-    """
-    dx = Data(fname=fname, clk_p=12.5e-9, nch=8, leakage=leakage,
-              gamma=gamma)
-    ph_times_m, A_em, ph_times_det = load_data_ordered16(
-        fname=fname, n_bytes_to_read=bytes_to_read, swap_D_A=swap_D_A)
-    dx.add(ph_times_m=ph_times_m, A_em=A_em, ALEX=False)
-    return dx
-
-def multispot48_simple(fname, leakage=0, gamma=1.,
-                       i_start=0, i_stop=None, debug=False):
-    """Load a 48-ch multispot file and return a Data() object.
-    """
-    dx = Data(fname=fname, clk_p=10e-9, nch=48, leakage=leakage, gamma=gamma)
-    ph_times_m, big_fifo, ch_fifo = load_manta_timestamps(
-        fname, i_start=i_start, i_stop=i_stop, debug=debug)
-    A_em = [True] * len(ph_times_m)
-    dx.add(ph_times_m=ph_times_m, A_em=A_em, ALEX=False)
-    big_fifo_full = np.array([b.any() for b in big_fifo]).any()
-    ch_fifo_full = np.array([b.any() for b in ch_fifo]).any()
-    if big_fifo_full:
-        print('WARNING: Big-FIFO full, flags saved in Data()')
-        dx.add(big_fifo=big_fifo)
-    if ch_fifo_full:
-        print('WARNING: CH-FIFO full, flags saved in Data()')
-        dx.add(ch_fifo=ch_fifo)
-    return dx
-
-def multispot48(fname, leakage=0, gamma=1., reprocess=False,
-                i_start=0, i_stop=None, debug=False):
-    """Load a 48-ch multispot file and return a Data() object.
-    """
-    basename, ext = os.path.splitext(fname)
-    fname_h5 = basename + '.hdf5'
-    fname_dat = basename + '.dat'
-
-    def load_dat_file():
-        pprint(' - Loading DAT file: %s ... ' % fname_dat)
-        # Load data from raw file and store it in a HDF5 file
-        data = load_xavier_manta_data(fname_dat, i_start=i_start,
-                                      i_stop=i_stop, debug=debug)
-        pprint('DONE.\n - Extracting timestamps and detectors ... ')
-        timestamps, det = get_timestamps_detectors(data, nbits=24)
-        pprint('DONE.\n - Processing and storing ... ')
-        ph_times_m, big_fifo, ch_fifo = process_store(
-            timestamps, det, out_fname=fname_h5, fifo_flag=True, debug=False)
-        pprint('DONE.\n')
-        return ph_times_m, big_fifo, ch_fifo
-
-    if not (os.path.isfile(fname_dat) or os.path.isfile(fname_h5)):
-        raise IOError('Data file "%s" not found' % basename)
-
-    if os.path.exists(fname_h5) and not reprocess:
-        # There is a HDF5 file
-        try:
-            pprint(' - Loading HDF5 file: %s ... ' % fname_h5)
-            ph_times_m, big_fifo, ch_fifo = \
-                load_manta_timestamps_pytables(fname_h5)
-            pprint('DONE.\n')
-        except tables.HDF5ExtError:
-            pprint('\n  Ops! File may be truncated.\n')
-            ph_times_m, big_fifo, ch_fifo = load_dat_file()
-    else:
-        ph_times_m, big_fifo, ch_fifo = load_dat_file()
-
-    # Current data has only acceptor ch
-    A_em = [True] * len(ph_times_m)
-
-    dx = Data(fname=fname, clk_p=10e-9, nch=48, leakage=leakage, gamma=gamma)
-    dx.add(ph_times_m=ph_times_m, A_em=A_em, ALEX=False,
-           data_file=ph_times_m.data_file, bg_data_file=ph_times_m.data_file)
-    big_fifo_full = np.array([b[:].any() for b in big_fifo]).any()
-    ch_fifo_full = np.array([b[:].any() for b in ch_fifo]).any()
-    if big_fifo_full:
-        print('WARNING: Big-FIFO full, flags saved in Data()')
-        dx.add(big_fifo=big_fifo)
-    if ch_fifo_full:
-        print('WARNING: CH-FIFO full, flags saved in Data()')
-        dx.add(ch_fifo=ch_fifo)
-    return dx
-
 
 ##
 # usALEX loader functions
@@ -398,8 +326,7 @@ def _select_range(times, period, edges):
     return _select_inner_range(times, period, edges) if edges[0] < edges[1] \
         else _select_outer_range(times, period, edges)
 
-def usalex(fname, leakage=0, gamma=1., header=None, start=None, stop=None,
-           BT=None):
+def usalex(fname, leakage=0, gamma=1., header=None, BT=None):
     """Load usALEX data from a SM file and return a Data() object.
 
     This function returns a Data() object to which you need to apply
@@ -420,14 +347,13 @@ def usalex(fname, leakage=0, gamma=1., header=None, start=None, stop=None,
     burst search, etc...
     """
     if BT is not None:
-        print('WARNING: `BT` argument is deprecated, use `leakage` instead.')
+        log.warning('`BT` argument is deprecated, use `leakage` instead.')
         leakage = BT
     if header is not None:
-        print('WARNING: `header` argument ignored. '
-              '         The header length is now computed automatically.')
+        log.warning('    `header` argument ignored. '
+                    '    The header length is now computed automatically.')
     print(" - Loading '%s' ... " % fname)
-    ph_times_t, det_t, labels = load_sm(fname, start=start, stop=stop,
-                                        return_labels=True)
+    ph_times_t, det_t, labels = load_sm(fname, return_labels=True)
     print(" [DONE]\n")
 
     DONOR_ON = (2850, 580)
@@ -435,41 +361,30 @@ def usalex(fname, leakage=0, gamma=1., header=None, start=None, stop=None,
     alex_period = 4000
 
     dx = Data(fname=fname, clk_p=12.5e-9, nch=1, leakage=leakage, gamma=gamma,
-              ALEX=True, lifetime=False,
+              ALEX=True, lifetime=False, alternated=True,
+              meas_type='smFRET-usALEX',
               D_ON=DONOR_ON, A_ON=ACCEPT_ON, alex_period=alex_period,
               ph_times_t=[ph_times_t], det_t=[det_t], det_donor_accept=(0, 1),
               ch_labels=labels)
     return dx
 
-def usalex_apply_period(d, delete_ph_t=True, remove_d_em_a_ex=False):
+
+def _usalex_apply_period_1ch(d, delete_ph_t=True, remove_d_em_a_ex=False,
+                             ich=0):
     """Applies to the Data object `d` the alternation period previously set.
 
-    Note that you first need to load the data in a variable `d` and then
-    set the alternation parameters using `d.add(D_ON=..., A_ON=...)`.
-
-    The typical pattern for loading ALEX data is the following::
-
-        d = loader.photon_hdf5(fname=fname)
-        d.add(D_ON=(2850, 580), A_ON=(900, 2580))
-        alex_plot_alternation(d)
-
-    If the plot looks good, apply the alternation with::
-
-        loader.alex_apply_period(d)
-
-    Now `d` is ready for futher processing such as background estimation,
-    burst search, etc...
-
-    *See also:* :func:`alex_apply_period`.
+    This function operates on a single-channel.
+    See :func:`usalex_apply_period` for details.
     """
-    ich = 0
     donor_ch, accept_ch = d._det_donor_accept_multich[ich]
     D_ON, A_ON = d._D_ON_multich[ich], d._A_ON_multich[ich]
     # Remove eventual ch different from donor or acceptor
-    d_ch_mask_t = (d.det_t[ich] == donor_ch)
-    a_ch_mask_t = (d.det_t[ich] == accept_ch)
+    det_t = d.det_t[ich][:]
+    ph_times_t = d.ph_times_t[ich][:]
+    d_ch_mask_t = (det_t == donor_ch)
+    a_ch_mask_t = (det_t == accept_ch)
     valid_mask = d_ch_mask_t + a_ch_mask_t
-    ph_times_val = d.ph_times_t[ich][valid_mask]
+    ph_times_val = ph_times_t[valid_mask]
     d_ch_mask_val = d_ch_mask_t[valid_mask]
     a_ch_mask_val = a_ch_mask_t[valid_mask]
     assert (d_ch_mask_val + a_ch_mask_val).all()
@@ -510,13 +425,19 @@ def usalex_apply_period(d, delete_ph_t=True, remove_d_em_a_ex=False):
     assert (d_em + a_em).all()       # masks fill the total array
     assert not (d_em * a_em).any()   # no photon is both D and A
     assert a_ex.size == a_em.size == d_ex.size == d_em.size == ph_times.size
-    d.add(ph_times_m=[ph_times],
-          D_em=[d_em], A_em=[a_em], D_ex=[d_ex], A_ex=[a_ex])
+    _append_data_ch(d, 'ph_times_m', ph_times)
+    _append_data_ch(d, 'D_em', d_em)
+    _append_data_ch(d, 'A_em', a_em)
+    _append_data_ch(d, 'D_ex', d_ex)
+    _append_data_ch(d, 'A_ex', a_ex)
+    assert (len(d.ph_times_m) == len(d.D_em) == len(d.A_em) ==
+            len(d.D_ex) == len(d.A_ex) == ich + 1)
 
     if 'particles_t' in d:
-        particles_val = d.particles_t[ich][valid_mask]
+        particles_t = d.particles_t[ich][:]
+        particles_val = particles_t[valid_mask]
         particles = particles_val[DexAex_mask]
-        d.add(particles=[particles])
+        _append_data_ch(d, 'particles', particles)
 
     assert d.ph_times_m[ich].size == d.A_em[ich].size
 
@@ -526,6 +447,35 @@ def usalex_apply_period(d, delete_ph_t=True, remove_d_em_a_ex=False):
     return d
 
 
+def usalex_apply_period(d, delete_ph_t=True, remove_d_em_a_ex=False):
+    """Applies to the Data object `d` the alternation period previously set.
+
+    Note that you first need to load the data in a variable `d` and then
+    set the alternation parameters using `d.add(D_ON=..., A_ON=...)`.
+
+    The typical pattern for loading ALEX data is the following::
+
+        d = loader.photon_hdf5(fname=fname)
+        d.add(D_ON=(2850, 580), A_ON=(900, 2580))
+        alex_plot_alternation(d)
+
+    If the plot looks good, apply the alternation with::
+
+        loader.alex_apply_period(d)
+
+    Now `d` is ready for futher processing such as background estimation,
+    burst search, etc...
+
+    *See also:* :func:`alex_apply_period`.
+    """
+    for ich in range(d.nch):
+        _usalex_apply_period_1ch(d, remove_d_em_a_ex=remove_d_em_a_ex, ich=ich,
+                                 delete_ph_t=False)
+    if delete_ph_t:
+        d.delete('ph_times_t')
+        d.delete('det_t')
+    return d
+
 ##
 # nsALEX loader functions
 #
@@ -564,6 +514,7 @@ def nsalex(fname):
               det_donor_accept=(4, 6))
     return dx
 
+
 def nsalex_apply_period(d, delete_ph_t=True):
     """Applies to the Data object `d` the alternation period previously set.
 
@@ -634,6 +585,7 @@ def nsalex_apply_period(d, delete_ph_t=True):
         d.delete('det_t')
         d.delete('nanotimes_t')
 
+
 def alex_apply_period(d, delete_ph_t=True):
     """Apply the ALEX period definition set in D_ON and A_ON attributes.
 
@@ -655,16 +607,20 @@ def alex_apply_period(d, delete_ph_t=True):
     Now `d` is ready for futher processing such as background estimation,
     burst search, etc...
     """
-    assert d.ALEX
+    assert d.alternated
     if d.lifetime:
         apply_period_func = nsalex_apply_period
     else:
         apply_period_func = usalex_apply_period
     apply_period_func(d, delete_ph_t=delete_ph_t)
-    msg = ('# Total photons (after ALEX selection):  {:10,}\n'
-           '#  D  photons in D+A excitation periods: {:10,}\n'
-           '#  A  photons in D+A excitation periods: {:10,}\n'
-           '# D+A photons in  D  excitation period:  {:10,}\n'
-           '# D+A photons in  A  excitation period:  {:10,}\n')
-    print(msg.format(d.ph_times_m[0].size, d.D_em[0].sum(), d.A_em[0].sum(),
-                     d.D_ex[0].sum(), d.A_ex[0].sum()))
+    msg = ('# Total photons (after ALEX selection):  {:12,}\n'
+           '#  D  photons in D+A excitation periods: {:12,}\n'
+           '#  A  photons in D+A excitation periods: {:12,}\n'
+           '# D+A photons in  D  excitation period:  {:12,}\n'
+           '# D+A photons in  A  excitation period:  {:12,}\n')
+    ph_data_size = d.ph_data_sizes.sum()
+    D_em_sum = sum(d_em.sum() for d_em in d.D_em)
+    A_em_sum = sum(a_em.sum() for a_em in d.A_em)
+    D_ex_sum = sum(d_ex.sum() for d_ex in d.D_ex)
+    A_ex_sum = sum(a_ex.sum() for a_ex in d.A_ex)
+    print(msg.format(ph_data_size, D_em_sum, A_em_sum, D_ex_sum, A_ex_sum))
